@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 1986, 1994, 1996, 2000, 2002
-**	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
+**	Jeff Forys (jeffware@marjum.com).  All rights reserved.
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that: (1) Redistributions of
@@ -11,7 +11,7 @@
 ** and/or other materials provided with the distribution, (3) All
 ** advertising materials mentioning features or use of this software
 ** must display the following acknowledgment: ``This product includes
-** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
+** software developed by Jeff Forys (jeffware@marjum.com).'', (4)
 ** The name of the author may not be used to endorse or promote products
 ** derived from this software without specific prior written permission.
 **
@@ -21,7 +21,7 @@
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: argparse.c,v 1.14 2002/04/05 18:23:16 forys Exp $";
+static char rcsid[] = "$Id: argparse.c,v 1.17 2005/04/06 23:43:25 forys Exp $";
 #endif
 
 #define	NO_AEXTERN
@@ -40,7 +40,7 @@ uid_T	*UidList = NULL;		/* list of user id's */
 pid_T	*PidList = NULL;		/* list of process id's */
 cmd_T	*CmdList = NULL;		/* list of commands */
 int	TtyIndx = 0, UidIndx = 0, PidIndx = 0, CmdIndx = 0;
-int	Fflag = 0, Iflag = 0, Nflag = 0, Vflag = 0, Wflag = 0;
+int	Aflag = 0, Fflag = 0, Iflag = 0, Nflag = 0, Vflag = 0, Wflag = 0;
 
 /*
  * Because some vendors now have long symbolic signal names, "kill -l" may
@@ -61,7 +61,7 @@ ArgParse(argc, argv)
 	char *argv[];
 {
 	extern int atoi();
-	extern char *calloc();
+	extern void *calloc();
 	struct passwd *pp;
 	struct stat st;
 	int argcnt = argc;	/* saved so ALLOC() knows max mem to allocate */
@@ -72,6 +72,16 @@ ArgParse(argc, argv)
 	if ((loc = (type) calloc((unsigned)cnt,sizeof(type))) == NULL)	\
 		Exceed(nstr)
 
+#ifndef	NO_GETENV
+	/* if "SKILL_VERBOSE" env var is set, enable verbose mode */
+	{
+		extern char *getenv();
+
+		if (getenv("SKILL_VERBOSE") != NULL)
+			Vflag++;
+	}
+#endif
+
 	/*
 	 * This is the main argument parsing loop.  At first glance
 	 * this may look confusing.  We destructively move across
@@ -99,6 +109,9 @@ ArgParse(argc, argv)
 		    do {
 			argloop = 0;
 			switch (*++(*argv)) {
+			    case 'a':
+				Aflag++;
+				break;
 			    case 'l':
 				if (Skill)
 					ListSigs();
@@ -263,7 +276,7 @@ ArgParse(argc, argv)
 	 *  the general user is not usually allowed to modify the
 	 *  state of other users' processes.
 	 */
-	if (UidIndx == 0 && MyUid != ROOTUID) {
+	if (Aflag == 0 && (UidIndx == 0 && MyUid != ROOTUID)) {
 		ALLOC(1, uid_T *, UidList, "user");
 		*(UidList + UidIndx++) = MyUid;
 	}
