@@ -1,6 +1,6 @@
 /*
-** Copyright (c) 1986, 1994, 1996, 2000, 2002
-**	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
+** Copyright (c) 1986, 1994, 1996, 2000, 2002, 2006
+**	Jeff Forys (jeffware@marjum.com).  All rights reserved.
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that: (1) Redistributions of
@@ -11,7 +11,7 @@
 ** and/or other materials provided with the distribution, (3) All
 ** advertising materials mentioning features or use of this software
 ** must display the following acknowledgment: ``This product includes
-** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
+** software developed by Jeff Forys (jeffware@marjum.com).'', (4)
 ** The name of the author may not be used to endorse or promote products
 ** derived from this software without specific prior written permission.
 **
@@ -21,7 +21,7 @@
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: aix-3.c,v 1.7 2002/03/21 18:20:44 forys Exp $";
+static char rcsid[] = "$Id: aix-3.c,v 1.10 2006/10/09 01:58:11 forys Exp $";
 #endif
 
 #define	NO_MEXTERN
@@ -33,6 +33,29 @@ static char rcsid[] = "$Id: aix-3.c,v 1.7 2002/03/21 18:20:44 forys Exp $";
 
 #include <stdio.h>
 
+/*
+ * For AIX 3 and AIX 4, #define the SKILL_AIX34 macro.
+ * For AIX 5 (and beyond), this file should compile correctly.
+ */
+#ifdef SKILL_AIX34
+#define	SKILL_P_STRUCT		struct procinfo
+#define	SKILL_P_STATE(p)	(p)->pi_stat
+#define	SKILL_P_FLAGS(p)	(p)->pi_flag
+#define	SKILL_PU_TTYP(p,u)	(u)->ui_ttyp
+#define	SKILL_PU_TTYD(p,u)	(u)->ui_ttyd
+#define	SKILL_PU_COMM(p,u)	(u)->ui_comm
+#define	SKILL_NOTTY		(-1)
+#else
+#define	SKILL_P_STRUCT		struct procentry64
+#define	SKILL_P_STATE(p)	(p)->pi_state
+#define	SKILL_P_FLAGS(p)	(p)->pi_flags
+#define	SKILL_PU_TTYP(p,u)	(p)->pi_ttyp
+#define	SKILL_PU_TTYD(p,u)	(p)->pi_ttyd
+#define	SKILL_PU_COMM(p,u)	(p)->pi_comm
+#define	SKILL_NOTTY		(0)
+#include <stdlib.h>
+#endif
+
 /*
  * Define SigNames, NSig, and TtyDevDir here; they are used by other
  * routines and must be global.  Everyone seems to have their own
@@ -49,7 +72,11 @@ char *SigMap[] = { "0",
 	"37", "38", "39", "40", "41", "42",			/* 37 - 42 */
 	"43", "44", "45", "46", "47", "48",			/* 43 - 48 */
 	"49", "50", "51", "52", "53", "54",			/* 49 - 54 */
+#ifdef	SKILL_AIX34
 	"55", "56", "57", "58", "59", "GRANT",			/* 55 - 60 */
+#else
+	"55", "56", "57", "58", "CPUFAIL", "GRANT",		/* 55 - 60 */
+#endif
 	"RETRACT", "SOUND", "SAK", "64",			/* 61 - 64 */
 };
 int NSig = NSIG;
@@ -138,8 +165,14 @@ MdepAction(pid)
 		return(setpriority(PRIO_PROCESS, (int)pid, SigPri));
 }
 
+#ifdef SKILL_AIX34
 /* disable regular expresions: regex(3) availability unknown */
 NULL_REGEX_FUNCS
+#else
+/* OS supports POSIX-style regular expressions */
+#include <regex.h>
+REAL_REGEX_FUNCS
+#endif
 
 /*
  * Now, set up everything we need to write a GetProc() routine.
@@ -174,7 +207,7 @@ NULL_REGEX_FUNCS
 
 extern	off_t lseek();
 
-#define	NPROCS	256	/* size of `procs' struct (if too small, use calloc) */
+#define	NPROCS	1024	/* size of `procs' struct (if too small, use calloc) */
 
 /*
  * GetProc()
@@ -191,13 +224,17 @@ GetProc()
 {
 	extern int errno;
 	extern char *SysErr();
-	static struct procinfo pinfo[NPROCS], *procsp;
+	static SKILL_P_STRUCT pinfo[NPROCS], *procsp;
+	register SKILL_P_STRUCT *aproc;
 	static struct ProcInfo procinfo;
-	register struct procinfo *aproc;
-	register struct userinfo *auser;
 	static int thisproc = 0;
 	static int initialized = 0;
 	static int needuser = 1;		/* Fflag support */
+#ifdef SKILL_AIX34
+	register struct userinfo *auser;
+#else
+	pid_t procsidx;
+#endif
 
 	/*
 	 * Read in all the processes at once, into a large block of memory;
@@ -215,8 +252,16 @@ GetProc()
 		if (initialized)
 			return((struct ProcInfo *)NULL);
 
-		pisize = sizeof(struct procinfo);
-		if ((thisproc = getproc(pinfo, (u_int)NPROCS, pisize)) < 0) {
+		pisize = sizeof(SKILL_P_STRUCT);
+#ifdef SKILL_AIX34
+		if ((thisproc = getproc(pinfo, (u_int)NPROCS, pisize)) < 0)
+#else
+		procsidx = 0;
+		if (((thisproc = getprocs64(pinfo, pisize, NULL, 0,
+				&procsidx, (u_int)NPROCS)) < 0) ||
+		    thisproc == NPROCS)
+#endif
+		{
 			/*
 			 * We apparently have more then NPROCS processes
 			 * running on this machine.  Ideally, one should
@@ -238,21 +283,27 @@ GetProc()
 			errno = nprocs;
 
 			nprocs = NPROCS;
-			while (errno == ENOSPC && nprocs <= maxnprocs) {
+			while (nprocs <= maxnprocs) {
 				nprocs <<= 1;
-				if ((procsp = (struct procinfo *)
+				if ((procsp = (SKILL_P_STRUCT *)
 				     calloc(nprocs, pisize)) == NULL) {
 					fprintf(stderr, errstr, ProgName,
 					        "getproc", "out of memory");
 					exit(EX_SERR);
 				}
 				errno = 0;
+#ifdef SKILL_AIX34
 				thisproc = getproc(procsp, nprocs, pisize);
+#else
+				procsidx = 0;
+				thisproc = getprocs64(procsp, pisize, NULL, 0,
+					&procsidx, nprocs);
+#endif
 				if (thisproc < 0 && errno != ENOSPC) {
 					fprintf(stderr, errstr, ProgName,
 						"getproc", SysErr());
 					exit(EX_SERR);
-				} else if (thisproc == nprocs)
+				} else if (thisproc < nprocs)
 					break;
 				free((void *)procsp);
 			}
@@ -281,7 +332,7 @@ GetProc()
 		aproc = procsp++;
 		thisproc--;
 
-		if (aproc->pi_stat != SNONE) {
+		if (SKILL_P_STATE(aproc) != SNONE) {
 			/*
 			 * Make sure this isn't a "zombie" or "exiting"
 			 * process.  If it is, we have all the information
@@ -291,11 +342,11 @@ GetProc()
 			procinfo.pi_pid = (pid_T) aproc->pi_pid;
 			procinfo.pi_uid = (uid_T) aproc->pi_uid;
 
-			if (aproc->pi_stat == SZOMB) {	/* zombie */
+			if (SKILL_P_STATE(aproc) == SZOMB) { /* zombie */
 				static char *zombie = "<defunct>";
 				procinfo.pi_flags |= PI_ZOMBIE;
 				procinfo.pi_cmd = zombie;
-			} else if (aproc->pi_flag & SEXIT) {
+			} else if (SKILL_P_FLAGS(aproc) & SEXIT) {
 				static char *exiting = "<exiting>";
 				procinfo.pi_flags |= PI_SWEXIT;
 				procinfo.pi_cmd = exiting;
@@ -306,31 +357,38 @@ GetProc()
 
 			if (procinfo.pi_flags || !needuser)
 				return(&procinfo);
+#ifdef SKILL_AIX34
 			else {
 				static struct userinfo uinfo;
-				if (getuser(aproc, sizeof(struct procinfo),
+				if (getuser(aproc, sizeof(SKILL_P_STRUCT),
 				            &uinfo,sizeof(struct userinfo)) < 0)
 					auser = NULL;
 				else
 					auser = &uinfo;
 			}
+#endif
 		}
 
-	} while (aproc->pi_stat == SNONE || auser == NULL);
+	}
+#ifdef SKILL_AIX34
+ 	while (SKILL_P_STATE(aproc) == SNONE || auser == NULL);
+#else
+ 	while (SKILL_P_STATE(aproc) == SNONE);
+#endif
 
 	/*
 	 * We now have a process (`aproc').
 	 * Fill in the rest of `procinfo'.
 	 */
-	if (auser->ui_ttyp != -1) {	/* has a controlling tty */
+	if (SKILL_PU_TTYP(aproc,auser) != SKILL_NOTTY) { /* controlling tty */
 		procinfo.pi_flags |= PI_CTLTTY;
-		procinfo.pi_tty = (tty_T) auser->ui_ttyd;
+		procinfo.pi_tty = (tty_T) SKILL_PU_TTYD(aproc,auser);
 	} else {
 		procinfo.pi_tty = (tty_T)-2;
 	}
 
 	/* set path-stripped command name */
-	SETCMD(procinfo.pi_cmd, auser->ui_comm, MAXCOMLEN)
+	SETCMD(procinfo.pi_cmd, SKILL_PU_COMM(aproc,auser), MAXCOMLEN)
 
 	return(&procinfo);
 }
