@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 1986, 1994, 1996, 2000, 2002
-**	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
+**	Jeff Forys (jeffware@marjum.com).  All rights reserved.
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that: (1) Redistributions of
@@ -11,7 +11,7 @@
 ** and/or other materials provided with the distribution, (3) All
 ** advertising materials mentioning features or use of this software
 ** must display the following acknowledgment: ``This product includes
-** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
+** software developed by Jeff Forys (jeffware@marjum.com).'', (4)
 ** The name of the author may not be used to endorse or promote products
 ** derived from this software without specific prior written permission.
 **
@@ -23,11 +23,26 @@
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: hpux-10.c,v 1.7 2002/04/05 03:10:26 forys Exp $";
+static char rcsid[] = "$Id: hpux-10.c,v 1.11 2005/04/06 23:49:34 forys Exp $";
 #endif
 
 #define	_XPG4_EXTENDED
 
+/* 3/22/05
+ *
+ * The 32-bit 'pst_status' struct returned by pstat_getproc(2) is not
+ * always large enough for 64-bit HP-UX values and the system call fails
+ * with errno set to EOVERFLOW.  The solution is to define _PSTAT64 to
+ * use the 64-bit sized structure.
+ *
+ * This change was tested on IA-64 superdome (11.23), PA-RISC 2.0 (11.11)
+ * and (32-bit) PA-RISC 1.1 (11.11).
+ *
+ * Thanks to Jerry Grooms <grooms@uky.edu> for finding this problem and
+ * for the resources to resolve it!
+ */
+#define	_PSTAT64
+
 #define	NO_MEXTERN
 #include "conf.h"
 #undef	NO_MEXTERN
@@ -176,7 +191,7 @@ REAL_REGEX_FUNCS
 
 #include <sys/pstat.h>
 
-#define	PROCSLOP	32	/* additional process slots to alloc */
+#define	PROCSLOP	256	/* additional process slots to alloc */
 
 /*
  * GetProc()
@@ -211,8 +226,18 @@ GetProc()
 			exit(2);
 		}
 		if ((pret=pstat_getproc(proclist,sizeof(*aproc),nprocs,0)) < 0){
+#ifndef _PSTAT64
+			extern int errno;
+			int saverrno = errno;
+#endif
 			fprintf(stderr, "%s: pstat_getproc: %s\n",
 				ProgName, SysErr());
+#ifndef _PSTAT64
+			if (saverrno == 72)	/* EOVERFLOW tweak */
+				fprintf(stderr,
+					"%s: recompile with \"-D_PSTAT64\"\n",
+					ProgName);
+#endif
 			exit(2);
 		}
 		aproc = proclist;
