@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 1986, 1994, 1996, 2000, 2002
-**	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
+**	Jeff Forys (jeffware@marjum.com).  All rights reserved.
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that: (1) Redistributions of
@@ -11,7 +11,7 @@
 ** and/or other materials provided with the distribution, (3) All
 ** advertising materials mentioning features or use of this software
 ** must display the following acknowledgment: ``This product includes
-** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
+** software developed by Jeff Forys (jeffware@marjum.com).'', (4)
 ** The name of the author may not be used to endorse or promote products
 ** derived from this software without specific prior written permission.
 **
@@ -23,7 +23,7 @@
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: linux-1.c,v 1.12 2002/04/05 03:10:26 forys Exp $";
+static char rcsid[] = "$Id: linux-1.c,v 1.15 2005/04/06 23:49:35 forys Exp $";
 #endif
 
 #define	NO_MEXTERN
@@ -32,13 +32,16 @@ static char rcsid[] = "$Id: linux-1.c,v 1.12 2002/04/05 03:10:26 forys Exp $";
 
 #include <sys/time.h>
 #include <sys/resource.h>	/* for [get|set]priority() */
+#ifndef	__CYGWIN__
 #include <linux/fs.h>		/* for MINOR() macro */
+#endif
 
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <errno.h>
 
 extern int MissedProcCnt;
 
@@ -104,8 +107,10 @@ MdepInit(pname)
 	 * If we are running as root, raise our priority to better
 	 * catch runaway processes.
 	 */
+#ifndef	__CYGWIN__
 	if (MyUid == ROOTUID)
 		(void) setpriority(PRIO_PROCESS, MyPid, PRIO_MIN);
+#endif
 
 	/*
 	 * Determine what we are doing to processes we find.  We will
@@ -118,10 +123,15 @@ MdepInit(pname)
 	 * Initialize SigPri to either default signal (`skill') or
 	 * default priority (`snice').
 	 */
+#ifndef	__CYGWIN__
 	PrioMin = PRIO_MIN;
 	PrioMax = PRIO_MAX;
+#else
+	PrioMin = PrioMax = 0;
+#endif
 	SigPri = Skill ? SIGTERM : 4;
 
+#ifndef	__CYGWIN__
 	/*
 	 * chdir to `TtyDevDir' to speed up tty argument parsing.
 	 */
@@ -130,6 +140,7 @@ MdepInit(pname)
 		        SysErr());
 		exit(EX_SERR);
 	}
+#endif
 }
 
 /*
@@ -147,7 +158,15 @@ MdepAction(pid)
 	if (Skill)
 		return(kill((int)pid, SigPri));
 	else
+#ifndef	__CYGWIN__
 		return(setpriority(PRIO_PROCESS, (int)pid, SigPri));
+#else
+		{
+			extern int errno;
+			errno = ENOSYS;
+			return -1;
+		}
+#endif
 }
 
 /* OS supports POSIX-style regular expressions */
