@@ -1,6 +1,6 @@
 /*
-** Copyright (c) 1986, 1994, 1996, 2000, 2002
-**	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
+** Copyright (c) 1986, 1994, 1996, 2000, 2002, 2006
+**	Jeff Forys (jeffware@marjum.com).  All rights reserved.
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that: (1) Redistributions of
@@ -11,7 +11,7 @@
 ** and/or other materials provided with the distribution, (3) All
 ** advertising materials mentioning features or use of this software
 ** must display the following acknowledgment: ``This product includes
-** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
+** software developed by Jeff Forys (jeffware@marjum.com).'', (4)
 ** The name of the author may not be used to endorse or promote products
 ** derived from this software without specific prior written permission.
 **
@@ -23,7 +23,7 @@
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: sys-5r4.c,v 1.19 2002/04/05 03:10:26 forys Exp $";
+static char rcsid[] = "$Id: sys-5r4.c,v 1.23 2006/10/09 01:59:56 forys Exp $";
 #endif
 
 #define	NO_MEXTERN
@@ -99,6 +99,15 @@ pid_T	MyPid;			/* pid of this process */
 uid_T	MyUid;			/* uid of this process */
 char	*ProgName;		/* program name */
 
+#ifdef sun
+int timeout;
+void
+alarmhdlr()
+{
+	timeout = 1;
+}
+#endif
+
 /*
  * This is the machine-dependent initialization routine.
  *
@@ -120,6 +129,7 @@ MdepInit(pname)
 
 #ifdef sun
 	{
+		struct sigaction avec;
 		extern int SigsPerLine;
 		int sig;
 
@@ -145,6 +155,11 @@ MdepInit(pname)
 				sprintf(SigMap[sig], "%d", sig);
 		}
 # endif
+		sigemptyset (&avec.sa_mask);
+		sigaddset (&avec.sa_mask, SIGALRM);
+		avec.sa_flags = 0;			/* no restart */
+		avec.sa_handler = alarmhdlr;
+		(void) sigaction(SIGALRM, &avec, NULL);
 	}
 #endif
 
@@ -239,7 +254,7 @@ GetProc()
 	int fd;
 
 	/*
-	 * If this is our first time here, open the proc directory,...
+	 * If this is our first time here, open the proc directory...
 	 */
 	if (dirfp == NULL && (dirfp=opendir(ProcDir)) == NULL) {
 		fprintf(stderr, "%s: %s: %s\n", ProgName, ProcDir, SysErr());
@@ -250,8 +265,20 @@ GetProc()
 		if (strcmp(dp->d_name,".") == 0 || strcmp(dp->d_name,"..") == 0)
 			continue;
 		(void) sprintf(flnm, ProcFil, dp->d_name);
-		if ((fd = open(flnm, O_RDONLY)) < 0) {
+#ifdef sun
+		timeout = 0;
+		alarm(2);
+#endif
+		fd = open(flnm, O_RDONLY);
+#ifdef sun
+		alarm(0);
+#endif
+		if (fd < 0) {
 			MissedProcCnt++;
+#ifdef sun
+			if (Wflag && timeout)
+				printf("Warning: open(%s) blocked\n", flnm);
+#endif
 			continue;	/* ignore procs we don't own */
 		}
 
