@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 1986, 1994, 1996, 2000, 2002
-**	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
+**	Jeff Forys (jeffware@marjum.com).  All rights reserved.
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that: (1) Redistributions of
@@ -11,7 +11,7 @@
 ** and/or other materials provided with the distribution, (3) All
 ** advertising materials mentioning features or use of this software
 ** must display the following acknowledgment: ``This product includes
-** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
+** software developed by Jeff Forys (jeffware@marjum.com).'', (4)
 ** The name of the author may not be used to endorse or promote products
 ** derived from this software without specific prior written permission.
 **
@@ -24,20 +24,20 @@
  * skill - send signals to processes by tty, user name, command or proc id.
  * snice - reprioritize processes by tty, user name, command or proc id.
  *
- * Version 4.1.0
+ * Version 4.1.4
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: main.c,v 1.36 2002/04/06 04:25:39 forys Exp $";
+static char rcsid[] = "$Id: main.c,v 1.42 2007/05/04 20:54:58 forys Exp $";
 /*
  * SCCS version release number is manually updated (for what(1), etc).
- * If you use SCCS, please use last extension for version (e.g. "4.1.0.1").
+ * If you use SCCS, please use last extension for version (e.g. "4.1.4.1").
  */
 static char sccsid[] = 
-	"@(#)skill	4.1.0.0 (jeff@forys.cranbury.nj.us) 04/07/02";
+	"@(#)skill	4.1.4.0 (jeffware@marjum.com) 05/04/07";
 #endif
 
-const char *Version = "4.1.0 20020407";
+const char *Version = "4.1.4 20070504";
 
 const char *CopyrightVersion = "%s %s\n\nCopyright (c) 1986, 1994, 1996, 2000, 2002 Jeff Forys.  All rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, is permitted under the terms specified in the source\ncode (and manual pages).\n";
 
@@ -150,6 +150,9 @@ main(argc, argv)
 		if (PID == MyPid || PID == 0)	/* ignore self */
 			continue;
 
+		if (Recorded(PID))		/* record/ignore duplicates */
+			continue;
+
 		if (Iflag || (!Nflag && (FLAGS & PI_ASKUSR))) {	/* ask user */
 			static char yesno[10];
 
@@ -278,6 +281,57 @@ WhoIs(uid)
 	return(usrcache);
 }
 
+/*
+ * Recorded(pid)
+ *
+ * Some OSes report the same PID multiple times, once per thread.
+ * Since there's no need to act on a process multiple times, this
+ * function is called to record that we are about to act on 'pid'.
+ *
+ * Return value: '1' if we've already acted on it, '0' if not.
+ *
+ * Note: caller ensures that we are never called with (pid == 0).
+ */
+int
+Recorded(pid)
+	pid_T pid;
+{
+	static pid_T prev_pid, *pptr_cur, pids[2048];
+	pid_T *pptr;
+
+	/*
+	 * Duplicates often come one after the other, so this simple
+	 * cache of the previous pid obviates a search through pids[].
+	 *
+	 * On these systems, even if pids[] becomes full, we'll still
+	 * not see duplicate pids since the cache continues to operate.
+	 */
+	if (prev_pid == pid)
+		return 1;
+	else
+		prev_pid = pid;
+
+	/*
+	 * If this is the first time, initialize pptr_cur pointer.
+	 * Else, search for 'pid' and return '1' if found.
+	 */
+	if (pptr_cur == NULL)		/* first time; init and assign pid */
+		pptr_cur = pids;
+	else
+		for (pptr = pids; pptr < pptr_cur; pptr++)
+			if (*pptr == pid)
+				return 1;
+
+	/*
+	 * Either "first time" or "not found" ... either way, copy
+	 * this pid to the end of pids[] (provided there is room).
+	 */
+	if (pptr_cur < &pids[sizeof(pids) / sizeof(pid_T)])
+		*pptr_cur++ = pid;
+
+	return 0;
+}
+
 /*
  * Usage(error)
  *
@@ -291,7 +345,7 @@ Usage(error)
 	switch (error) {
 	    case E_USAGE:
 		fprintf(stderr,
-		        "Usage: %s [%s] [-ivfwn] {<tty> <user> <pid> <cmd>}\n",
+		        "Usage: %s [%s] [-afinvw] {<tty> <user> <pid> <cmd>}\n",
 		        ProgName, Skill? "-<signal>": "(+|-)<priority>");
 		break;
 	    case E_PRIOR:	/* unused... remains for posterity? */
